@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2018 Red Hat Inc.
+ * Copyright (C) 2018-2020 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -51,6 +51,7 @@
 
 #include "bitmap.h"
 #include "minmax.h"
+#include "utils.h"
 
 #include "cache.h"
 #include "blk.h"
@@ -109,8 +110,18 @@ blk_init (void)
 #ifdef HAVE_MKOSTEMP
   fd = mkostemp (template, O_CLOEXEC);
 #else
+  /* Not atomic, but this is only invoked during .load, so the race
+   * won't affect any plugin actions trying to fork
+   */
   fd = mkstemp (template);
-  fcntl (fd, F_SETFD, FD_CLOEXEC);
+  if (fd >= 0) {
+    fd = set_cloexec (fd);
+    if (fd < 0) {
+      int e = errno;
+      unlink (template);
+      errno = e;
+    }
+  }
 #endif
   if (fd == -1) {
     nbdkit_error ("mkostemp: %s: %m", tmpdir);
@@ -190,8 +201,6 @@ blk_read (struct nbdkit_next_ops *next_ops, void *nxdata,
 
     /* If cache-on-read, copy the block to the cache. */
     if (cache_on_read) {
-      off_t offset = blknum * blksize;
-
       nbdkit_debug ("cache: cache-on-read block %" PRIu64
                     " (offset %" PRIu64 ")",
                     blknum, (uint64_t) offset);
@@ -217,6 +226,52 @@ blk_read (struct nbdkit_next_ops *next_ops, void *nxdata,
   }
 }
 
+int
+blk_cache (struct nbdkit_next_ops *next_ops, void *nxdata,
+           uint64_t blknum, uint8_t *block, int *err)
+{
+  off_t offset = blknum * blksize;
+  enum bm_entry state = bitmap_get_blk (&bm, blknum, BLOCK_NOT_CACHED);
+
+  reclaim (fd, &bm);
+
+  nbdkit_debug ("cache: blk_cache block %" PRIu64 " (offset %" PRIu64 ") is %s",
+                blknum, (uint64_t) offset,
+                state == BLOCK_NOT_CACHED ? "not cached" :
+                state == BLOCK_CLEAN ? "clean" :
+                state == BLOCK_DIRTY ? "dirty" :
+                "unknown");
+
+  if (state == BLOCK_NOT_CACHED) {
+    /* Read underlying plugin, copy to cache regardless of cache-on-read. */
+    if (next_ops->pread (nxdata, block, blksize, offset, 0, err) == -1)
+      return -1;
+
+    nbdkit_debug ("cache: cache block %" PRIu64 " (offset %" PRIu64 ")",
+                  blknum, (uint64_t) offset);
+
+    if (pwrite (fd, block, blksize, offset) == -1) {
+      *err = errno;
+      nbdkit_error ("pwrite: %m");
+      return -1;
+    }
+    bitmap_set_blk (&bm, blknum, BLOCK_CLEAN);
+    lru_set_recently_accessed (blknum);
+  }
+  else {
+#if HAVE_POSIX_FADVISE
+    int r = posix_fadvise (fd, offset, blksize, POSIX_FADV_WILLNEED);
+    if (r) {
+      errno = r;
+      nbdkit_error ("posix_fadvise: %m");
+      return -1;
+    }
+#endif
+    lru_set_recently_accessed (blknum);
+  }
+  return 0;
+}
+
 int
 blk_writethrough (struct nbdkit_next_ops *next_ops, void *nxdata,
                   uint64_t blknum, const uint8_t *block, uint32_t flags,
