@@ -138,9 +138,12 @@ struct nbdkit_plugin {
   int (*preconnect) (int readonly);
 
   int (*get_ready) (void);
+  int (*after_fork) (void);
 };
 
-extern void nbdkit_set_error (int err);
+NBDKIT_EXTERN_DECL (void, nbdkit_set_error, (int err));
+NBDKIT_EXTERN_DECL (const char *, nbdkit_export_name, (void));
+NBDKIT_EXTERN_DECL (int, nbdkit_is_tls, (void));
 
 #define NBDKIT_REGISTER_PLUGIN(plugin)                                  \
   NBDKIT_CXX_LANG_C                                                     \
