@@ -38,58 +38,44 @@
 #include <inttypes.h>
 #include <string.h>
 
-#include <pthread.h>
-
 #if defined(HAVE_GNUTLS) && defined(HAVE_GNUTLS_BASE64_DECODE2)
 #include <gnutls/gnutls.h>
 #endif
 
 #define NBDKIT_API_VERSION 2
-
 #include <nbdkit-plugin.h>
 
 #include "cleanup.h"
-#include "sparse.h"
+#include "allocator.h"
 #include "format.h"
 
-/* If raw|base64|data parameter seen. */
-static int data_seen = 0;
+/* Data (raw|base64|data) parameter. */
+static enum { NOT_SEEN, RAW, BASE64, DATA } data_seen = NOT_SEEN;
+static const char *data_param;
 
-/* size= parameter on the command line, -1 if not set. */
+/* size= parameter on the command line.  After configuration, it is
+ * set to the final size.
+ */
 static int64_t size = -1;
 
-/* Size of data specified on the command line. */
-static int64_t data_size = -1;
-
-/* Sparse array - the lock must be held when accessing this from
- * connected callbacks.
- */
-static struct sparse_array *sa;
-static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
+/* Allocator. */
+static struct allocator *a;
+const char *allocator_type = "sparse";
 
 /* Debug directory operations (-D data.dir=1). */
 int data_debug_dir;
 
-static void
-data_load (void)
-{
-  sa = alloc_sparse_array (data_debug_dir);
-  if (sa == NULL) {
-    perror ("malloc");
-    exit (EXIT_FAILURE);
-  }
-}
-
 /* On unload, free the sparse array. */
 static void
 data_unload (void)
 {
-  free_sparse_array (sa);
+  if (a)
+    a->f->free (a);
 }
 
 /* Parse the base64 parameter. */
 static int
-read_base64 (const char *value, int64_t *size_ret)
+read_base64 (const char *value, uint64_t *size_ret)
 {
 #if defined(HAVE_GNUTLS) && defined(HAVE_GNUTLS_BASE64_DECODE2)
   gnutls_datum_t in, out;
@@ -103,7 +89,7 @@ read_base64 (const char *value, int64_t *size_ret)
     return -1;
   }
 
-  if (sparse_array_write (sa, out.data, out.size, 0) == -1)
+  if (a->f->write (a, out.data, out.size, 0) == -1)
     return -1;
   free (out.data);
   *size_ret = out.size;
@@ -125,30 +111,27 @@ data_config (const char *key, const char *value)
       return -1;
     size = r;
   }
-  else if (strcmp (key, "raw") == 0 ||
-           strcmp (key, "base64") == 0 ||
-           strcmp (key, "data") == 0) {
-    if (data_seen) {
+  else if (strcmp (key, "allocator") == 0) {
+    allocator_type = value;
+  }
+  else if (strcmp (key, "raw") == 0) {
+    if (data_seen != NOT_SEEN) {
+    seen_error:
       nbdkit_error ("raw|base64|data parameter must be specified exactly once");
       return -1;
     }
-    data_seen = 1;
-
-    if (strcmp (key, "raw") == 0) {
-      data_size = strlen (value);
-      if (sparse_array_write (sa, value, data_size, 0) == -1)
-        return -1;
-    }
-    else if (strcmp (key, "base64") == 0) {
-      if (read_base64 (value, &data_size) == -1)
-        return -1;
-    }
-    else if (strcmp (key, "data") == 0) {
-      if (read_data_format (value, sa, &data_size) == -1)
-        return -1;
-    }
-    else
-      abort (); /* cannot happen */
+    data_seen = RAW;
+    data_param = value;
+  }
+  else if (strcmp (key, "base64") == 0) {
+    if (data_seen != NOT_SEEN) goto seen_error;
+    data_seen = BASE64;
+    data_param = value;
+  }
+  else if (strcmp (key, "data") == 0) {
+    if (data_seen != NOT_SEEN) goto seen_error;
+    data_seen = DATA;
+    data_param = value;
   }
   else {
     nbdkit_error ("unknown parameter '%s'", key);
@@ -158,15 +141,54 @@ data_config (const char *key, const char *value)
   return 0;
 }
 
-/* Check the raw|base64|data was specified, and set the final size. */
+/* Check the raw|base64|data was specified. */
 static int
 data_config_complete (void)
 {
-  if (!data_seen) {
+  if (data_seen == NOT_SEEN) {
     nbdkit_error ("raw|base64|data parameter was not specified");
     return -1;
   }
 
+  return 0;
+}
+
+#define data_config_help \
+  "data|raw|base64=...     Specify disk data on the command line\n" \
+  "size=<SIZE>             Size of the backing disk"
+
+/* Parse raw|base64|data parameter and set the final size. */
+static int
+data_get_ready (void)
+{
+  uint64_t data_size = 0; /* Size of data specified on the command line. */
+
+  a = create_allocator (allocator_type, data_debug_dir);
+  if (a == NULL)
+    return -1;
+
+  switch (data_seen) {
+  case RAW:
+    data_size = strlen (data_param);
+    if (a->f->write (a, data_param, data_size, 0) == -1)
+      return -1;
+    break;
+
+  case BASE64:
+    if (read_base64 (data_param, &data_size) == -1)
+      return -1;
+    break;
+
+  case DATA:
+    if (read_data_format (data_param, a, &data_size) == -1)
+      return -1;
+    break;
+
+  case NOT_SEEN:
+  default:
+    abort ();
+  }
+
   nbdkit_debug ("implicit data size: %" PRIi64, data_size);
 
   /* If size == -1 it means the size= parameter was not given so we
@@ -176,13 +198,12 @@ data_config_complete (void)
     size = data_size;
   nbdkit_debug ("final size: %" PRIi64, size);
 
+  if (a->f->set_size_hint (a, size) == -1)
+    return -1;
+
   return 0;
 }
 
-#define data_config_help \
-  "data|raw|base64=...     Specify disk data on the command line\n" \
-  "size=<SIZE>             Size of the backing disk"
-
 /* Provide a way to detect if the base64 feature is supported. */
 static void
 data_dump_plugin (void)
@@ -190,6 +211,16 @@ data_dump_plugin (void)
 #if defined(HAVE_GNUTLS) && defined(HAVE_GNUTLS_BASE64_DECODE2)
   printf ("data_base64=yes\n");
 #endif
+#ifdef HAVE_MLOCK
+  printf ("mlock=yes\n");
+#else
+  printf ("mlock=no\n");
+#endif
+#ifdef HAVE_LIBZSTD
+  printf ("zstd=yes\n");
+#else
+  printf ("zstd=no\n");
+#endif
 }
 
 #define THREAD_MODEL NBDKIT_THREAD_MODEL_PARALLEL
@@ -245,9 +276,7 @@ data_pread (void *handle, void *buf, uint32_t count, uint64_t offset,
             uint32_t flags)
 {
   assert (!flags);
-  ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&lock);
-  sparse_array_read (sa, buf, count, offset);
-  return 0;
+  return a->f->read (a, buf, count, offset);
 }
 
 /* Write data. */
@@ -257,8 +286,7 @@ data_pwrite (void *handle, const void *buf, uint32_t count, uint64_t offset,
 {
   /* Flushing, and thus FUA flag, is a no-op */
   assert ((flags & ~NBDKIT_FLAG_FUA) == 0);
-  ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&lock);
-  return sparse_array_write (sa, buf, count, offset);
+  return a->f->write (a, buf, count, offset);
 }
 
 /* Zero. */
@@ -266,12 +294,10 @@ static int
 data_zero (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
 {
   /* Flushing, and thus FUA flag, is a no-op. Assume that
-   * sparse_array_zero generally beats writes, so FAST_ZERO is a no-op. */
+   * a->f->zero generally beats writes, so FAST_ZERO is a no-op. */
   assert ((flags & ~(NBDKIT_FLAG_FUA | NBDKIT_FLAG_MAY_TRIM |
                      NBDKIT_FLAG_FAST_ZERO)) == 0);
-  ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&lock);
-  sparse_array_zero (sa, count, offset);
-  return 0;
+  return a->f->zero (a, count, offset);
 }
 
 /* Trim (same as zero). */
@@ -280,9 +306,7 @@ data_trim (void *handle, uint32_t count, uint64_t offset, uint32_t flags)
 {
   /* Flushing, and thus FUA flag, is a no-op */
   assert ((flags & ~NBDKIT_FLAG_FUA) == 0);
-  ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&lock);
-  sparse_array_zero (sa, count, offset);
-  return 0;
+  return a->f->zero (a, count, offset);
 }
 
 /* Nothing is persistent, so flush is trivially supported */
@@ -297,19 +321,19 @@ static int
 data_extents (void *handle, uint32_t count, uint64_t offset,
               uint32_t flags, struct nbdkit_extents *extents)
 {
-  ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&lock);
-  return sparse_array_extents (sa, count, offset, extents);
+  return a->f->extents (a, count, offset, extents);
 }
 
 static struct nbdkit_plugin plugin = {
   .name              = "data",
   .version           = PACKAGE_VERSION,
-  .load              = data_load,
   .unload            = data_unload,
   .config            = data_config,
   .config_complete   = data_config_complete,
   .config_help       = data_config_help,
+  .magic_config_key  = "data",
   .dump_plugin       = data_dump_plugin,
+  .get_ready         = data_get_ready,
   .open              = data_open,
   .get_size          = data_get_size,
   .can_multi_conn    = data_can_multi_conn,
