@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2018 Red Hat Inc.
+ * Copyright (C) 2018-2020 Red Hat Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -48,10 +48,12 @@
 
 #include <nbdkit-plugin.h>
 
+#include "ascii-string.h"
 #include "byte-swapping.h"
 #include "isaligned.h"
 #include "iszero.h"
 #include "rounding.h"
+#include "vector.h"
 
 #include "random.h"
 #include "regions.h"
@@ -68,22 +70,22 @@ int partitioning_debug_regions;
  * following partitions.
  */
 unsigned long alignment = DEFAULT_ALIGNMENT;
-int mbr_id = DEFAULT_MBR_ID;
+uint8_t mbr_id = DEFAULT_MBR_ID;
 char type_guid[16]; /* initialized by partitioning_load function below */
 
 /* partition-type parameter. */
 int parttype = PARTTYPE_UNSET;
 
 /* Files supplied on the command line. */
-struct file *files = NULL;
-size_t nr_files = 0;
+files the_files;
 
 /* Virtual disk layout. */
-struct regions regions;
+regions the_regions;
 
 /* Primary and secondary partition tables and extended boot records.
  * Secondary PT is only used for GPT.  EBR array of sectors is only
- * used for MBR with > 4 partitions and has length equal to nr_files-3.
+ * used for MBR with > 4 partitions and has length equal to
+ * the_files.size-3.
  */
 unsigned char *primary = NULL, *secondary = NULL, **ebr = NULL;
 
@@ -93,7 +95,7 @@ static struct random_state random_state;
 static void
 partitioning_load (void)
 {
-  init_regions (&regions);
+  init_regions (&the_regions);
   parse_guid (DEFAULT_TYPE_GUID, type_guid);
   xsrandom (time (NULL), &random_state);
 }
@@ -103,19 +105,19 @@ partitioning_unload (void)
 {
   size_t i;
 
-  for (i = 0; i < nr_files; ++i)
-    close (files[i].fd);
-  free (files);
+  for (i = 0; i < the_files.size; ++i)
+    close (the_files.ptr[i].fd);
+  free (the_files.ptr);
 
   /* We don't need to free regions.regions[].u.data because it points
    * to primary, secondary or ebr which we free here.
    */
-  free_regions (&regions);
+  free_regions (&the_regions);
 
   free (primary);
   free (secondary);
   if (ebr) {
-    for (i = 0; i < nr_files-3; ++i)
+    for (i = 0; i < the_files.size-3; ++i)
       free (ebr[i]);
     free (ebr);
   }
@@ -124,7 +126,6 @@ partitioning_unload (void)
 static int
 partitioning_config (const char *key, const char *value)
 {
-  struct file *p;
   struct file file;
   size_t i;
   int err;
@@ -163,22 +164,19 @@ partitioning_config (const char *key, const char *value)
     for (i = 0; i < 16; ++i)
       file.guid[i] = xrandom (&random_state) & 0xff;
 
-    p = realloc (files, (nr_files+1) * sizeof (struct file));
-    if (p == NULL) {
+    if (files_append (&the_files, file) == -1) {
       err = errno;
       close (file.fd);
       errno = err;
       nbdkit_error ("realloc: %m");
       return -1;
     }
-    files = p;
-    files[nr_files] = file;
-    nr_files++;
   }
   else if (strcmp (key, "partition-type") == 0) {
-    if (strcasecmp (value, "mbr") == 0 || strcasecmp (value, "dos") == 0)
+    if (ascii_strcasecmp (value, "mbr") == 0 ||
+        ascii_strcasecmp (value, "dos") == 0)
       parttype = PARTTYPE_MBR;
-    else if (strcasecmp (value, "gpt") == 0)
+    else if (ascii_strcasecmp (value, "gpt") == 0)
       parttype = PARTTYPE_GPT;
     else {
       nbdkit_error ("unknown partition-type: %s", value);
@@ -209,13 +207,15 @@ partitioning_config (const char *key, const char *value)
     alignment = r;
   }
   else if (strcmp (key, "mbr-id") == 0) {
-    if (sscanf (value, "%i", &mbr_id) != 1) {
-      nbdkit_error ("could not parse mbr-id: %s", value);
+    if (ascii_strcasecmp (value, "default") == 0)
+      mbr_id = DEFAULT_MBR_ID;
+    else if (nbdkit_parse_uint8_t ("mbr-id", value, &mbr_id) == -1)
       return -1;
-    }
   }
   else if (strcmp (key, "type-guid") == 0) {
-    if (parse_guid (value, type_guid) == -1) {
+    if (ascii_strcasecmp (value, "default") == 0)
+      parse_guid (DEFAULT_TYPE_GUID, type_guid);
+    else if (parse_guid (value, type_guid) == -1) {
       nbdkit_error ("could not validate GUID: %s", value);
       return -1;
     }
@@ -236,19 +236,19 @@ partitioning_config_complete (void)
   bool needs_gpt;
 
   /* Not enough / too many files? */
-  if (nr_files == 0) {
+  if (the_files.size == 0) {
     nbdkit_error ("at least one file= parameter must be supplied");
     return -1;
   }
 
   total_size = 0;
-  for (i = 0; i < nr_files; ++i)
-    total_size += files[i].statbuf.st_size;
+  for (i = 0; i < the_files.size; ++i)
+    total_size += the_files.ptr[i].statbuf.st_size;
   needs_gpt = total_size > MAX_MBR_DISK_SIZE;
 
   /* Choose default parttype if not set. */
   if (parttype == PARTTYPE_UNSET) {
-    if (needs_gpt || nr_files > 4) {
+    if (needs_gpt || the_files.size > 4) {
       parttype = PARTTYPE_GPT;
       nbdkit_debug ("picking partition type GPT");
     }
@@ -263,17 +263,23 @@ partitioning_config_complete (void)
                   "but you requested %zu partition(s) "
                   "and a total size of %" PRIu64 " bytes (> %" PRIu64 ").  "
                   "Try using: partition-type=gpt",
-                  nr_files, total_size, (uint64_t) MAX_MBR_DISK_SIZE);
+                  the_files.size, total_size, (uint64_t) MAX_MBR_DISK_SIZE);
     return -1;
   }
 
-  return create_virtual_disk_layout ();
+  return 0;
 }
 
 #define partitioning_config_help \
   "file=<FILENAME>  (required) File(s) containing partitions\n" \
   "partition-type=mbr|gpt      Partition type"
 
+static int
+partitioning_get_ready (void)
+{
+  return create_virtual_disk_layout ();
+}
+
 /* Create the per-connection handle. */
 static void *
 partitioning_open (int readonly)
@@ -287,7 +293,7 @@ partitioning_open (int readonly)
 static int64_t
 partitioning_get_size (void *handle)
 {
-  return virtual_size (&regions);
+  return virtual_size (&the_regions);
 }
 
 /* Serves the same data over multiple connections. */
@@ -297,12 +303,20 @@ partitioning_can_multi_conn (void *handle)
   return 1;
 }
 
+/* Cache. */
+static int
+partitioning_can_cache (void *handle)
+{
+  /* Let nbdkit call pread to populate the file system cache. */
+  return NBDKIT_CACHE_EMULATE;
+}
+
 /* Read data. */
 static int
 partitioning_pread (void *handle, void *buf, uint32_t count, uint64_t offset)
 {
   while (count > 0) {
-    const struct region *region = find_region (&regions, offset);
+    const struct region *region = find_region (&the_regions, offset);
     size_t i, len;
     ssize_t r;
 
@@ -314,14 +328,15 @@ partitioning_pread (void *handle, void *buf, uint32_t count, uint64_t offset)
     switch (region->type) {
     case region_file:
       i = region->u.i;
-      assert (i < nr_files);
-      r = pread (files[i].fd, buf, len, offset - region->start);
+      assert (i < the_files.size);
+      r = pread (the_files.ptr[i].fd, buf, len, offset - region->start);
       if (r == -1) {
-        nbdkit_error ("pread: %s: %m", files[i].filename);
+        nbdkit_error ("pread: %s: %m", the_files.ptr[i].filename);
         return -1;
       }
       if (r == 0) {
-        nbdkit_error ("pread: %s: unexpected end of file", files[i].filename);
+        nbdkit_error ("pread: %s: unexpected end of file",
+                      the_files.ptr[i].filename);
         return -1;
       }
       len = r;
@@ -350,7 +365,7 @@ partitioning_pwrite (void *handle,
                      const void *buf, uint32_t count, uint64_t offset)
 {
   while (count > 0) {
-    const struct region *region = find_region (&regions, offset);
+    const struct region *region = find_region (&the_regions, offset);
     size_t i, len;
     ssize_t r;
 
@@ -362,10 +377,10 @@ partitioning_pwrite (void *handle,
     switch (region->type) {
     case region_file:
       i = region->u.i;
-      assert (i < nr_files);
-      r = pwrite (files[i].fd, buf, len, offset - region->start);
+      assert (i < the_files.size);
+      r = pwrite (the_files.ptr[i].fd, buf, len, offset - region->start);
       if (r == -1) {
-        nbdkit_error ("pwrite: %s: %m", files[i].filename);
+        nbdkit_error ("pwrite: %s: %m", the_files.ptr[i].filename);
         return -1;
       }
       len = r;
@@ -404,8 +419,8 @@ partitioning_flush (void *handle)
 {
   size_t i;
 
-  for (i = 0; i < nr_files; ++i) {
-    if (fdatasync (files[i].fd) == -1) {
+  for (i = 0; i < the_files.size; ++i) {
+    if (fdatasync (the_files.ptr[i].fd) == -1) {
       nbdkit_error ("fdatasync: %m");
       return -1;
     }
@@ -423,9 +438,11 @@ static struct nbdkit_plugin plugin = {
   .config_complete   = partitioning_config_complete,
   .config_help       = partitioning_config_help,
   .magic_config_key = "file",
+  .get_ready         = partitioning_get_ready,
   .open              = partitioning_open,
   .get_size          = partitioning_get_size,
   .can_multi_conn    = partitioning_can_multi_conn,
+  .can_cache         = partitioning_can_cache,
   .pread             = partitioning_pread,
   .pwrite            = partitioning_pwrite,
   .flush             = partitioning_flush,
