@@ -43,8 +43,9 @@
 #include <dlfcn.h>
 #include <libgen.h>
 
-#define NBDKIT_API_VERSION 2
+#include <pthread.h>
 
+#define NBDKIT_API_VERSION 2
 #include <nbdkit-plugin.h>
 
 #include "cleanup.h"
@@ -75,6 +76,7 @@ int vddk_debug_datapath = 1;
 
 static void *dl;                           /* dlopen handle */
 static bool init_called;                   /* was InitEx called */
+static __thread int error_suppression;     /* threadlocal error suppression */
 
 static char *config;                       /* config */
 static const char *cookie;                 /* cookie */
@@ -107,6 +109,21 @@ static bool is_remote;
   if (vddk_debug_datapath)                                      \
     nbdkit_debug ("VDDK call: %s (" fs ")", fn, ##__VA_ARGS__)
 
+/* Unload the plugin. */
+static void
+vddk_unload (void)
+{
+  if (init_called) {
+    DEBUG_CALL ("VixDiskLib_Exit", "");
+    VixDiskLib_Exit ();
+  }
+  if (dl)
+    dlclose (dl);
+  free (config);
+  free (libdir);
+  free (password);
+}
+
 static void
 trim (char *str)
 {
@@ -138,6 +155,11 @@ error_function (const char *fs, va_list args)
 {
   CLEANUP_FREE char *str = NULL;
 
+  /* If the thread-local error_suppression flag is non-zero then we
+   * will suppress error messages from VDDK in this thread.
+   */
+  if (error_suppression) return;
+
   if (vasprintf (&str, fs, args) == -1) {
     nbdkit_error ("lost error message: %s", fs);
     return;
@@ -157,21 +179,6 @@ error_function (const char *fs, va_list args)
   nbdkit_error ("%s", str);
 }
 
-/* Unload the plugin. */
-static void
-vddk_unload (void)
-{
-  if (init_called) {
-    DEBUG_CALL ("VixDiskLib_Exit", "");
-    VixDiskLib_Exit ();
-  }
-  if (dl)
-    dlclose (dl);
-  free (config);
-  free (libdir);
-  free (password);
-}
-
 /* Configuration. */
 static int
 vddk_config (const char *key, const char *value)
@@ -400,12 +407,31 @@ vddk_config_complete (void)
   return 0;
 }
 
+#define vddk_config_help \
+  "[file=]<FILENAME>   (required) The filename (eg. VMDK file) to serve.\n" \
+  "Many optional parameters are supported, see nbdkit-vddk-plugin(3)."
+
 static int
 vddk_get_ready (void)
 {
-  VixError err;
-
   load_library (true);
+  return 0;
+}
+
+/* Defer VDDK initialization until after fork because it is known to
+ * create background threads from VixDiskLib_InitEx.  Unfortunately
+ * error reporting from this callback is difficult, but we have
+ * already checked in .get_ready that the library is dlopenable.
+ *
+ * For various hangs and failures which were caused by background
+ * threads and fork see:
+ * https://bugzilla.redhat.com/show_bug.cgi?id=1846309#c9
+ * https://www.redhat.com/archives/libguestfs/2019-April/msg00090.html
+ */
+static int
+vddk_after_fork (void)
+{
+  VixError err;
 
   /* Initialize VDDK library. */
   DEBUG_CALL ("VixDiskLib_InitEx",
@@ -426,10 +452,6 @@ vddk_get_ready (void)
   return 0;
 }
 
-#define vddk_config_help \
-  "[file=]<FILENAME>   (required) The filename (eg. VMDK file) to serve.\n" \
-  "Many optional parameters are supported, see nbdkit-vddk-plugin(3)."
-
 static void
 vddk_dump_plugin (void)
 {
@@ -456,11 +478,18 @@ vddk_dump_plugin (void)
 #endif
 }
 
-/* XXX To really do threading correctly in accordance with the VDDK
- * documentation, we must do all open/close calls from a single
- * thread.  This is a huge pain.
+/* The rules on threads and VDDK are here:
+ * https://code.vmware.com/docs/11750/virtual-disk-development-kit-programming-guide/GUID-6BE903E8-DC70-46D9-98E4-E34A2002C2AD.html
+ *
+ * Before nbdkit 1.22 we used SERIALIZE_ALL_REQUESTS.  Since nbdkit
+ * 1.22 we changed this to SERIALIZE_REQUESTS and added a mutex around
+ * calls to VixDiskLib_Open and VixDiskLib_Close.  This is not quite
+ * within the letter of the rules, but is within the spirit.
  */
-#define THREAD_MODEL NBDKIT_THREAD_MODEL_SERIALIZE_ALL_REQUESTS
+#define THREAD_MODEL NBDKIT_THREAD_MODEL_SERIALIZE_REQUESTS
+
+/* Lock protecting open/close calls - see above. */
+static pthread_mutex_t open_close_lock = PTHREAD_MUTEX_INITIALIZER;
 
 /* The per-connection handle. */
 struct vddk_handle {
@@ -502,6 +531,7 @@ free_connect_params (VixDiskLibConnectParams *params)
 static void *
 vddk_open (int readonly)
 {
+  ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&open_close_lock);
   struct vddk_handle *h;
   VixError err;
   uint32_t flags;
@@ -594,6 +624,7 @@ vddk_open (int readonly)
 static void
 vddk_close (void *handle)
 {
+  ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&open_close_lock);
   struct vddk_handle *h = handle;
 
   DEBUG_CALL ("VixDiskLib_Close", "handle");
@@ -765,6 +796,11 @@ vddk_can_extents (void *handle)
     return 0;
   }
 
+  /* Suppress errors around this call.  See:
+   * https://bugzilla.redhat.com/show_bug.cgi?id=1709211#c7
+   */
+  error_suppression = 1;
+
   /* However even when the call is available it rarely works well so
    * the best thing we can do here is to try the call and if it's
    * non-functional return false.
@@ -776,6 +812,7 @@ vddk_can_extents (void *handle)
                                          0, VIXDISKLIB_MIN_CHUNK_SIZE,
                                          VIXDISKLIB_MIN_CHUNK_SIZE,
                                          &block_list);
+  error_suppression = 0;
   if (err == VIX_OK) {
     DEBUG_CALL ("VixDiskLib_FreeBlockList", "block_list");
     VixDiskLib_FreeBlockList (block_list);
@@ -921,6 +958,7 @@ static struct nbdkit_plugin plugin = {
   .magic_config_key  = "file",
   .dump_plugin       = vddk_dump_plugin,
   .get_ready         = vddk_get_ready,
+  .after_fork        = vddk_after_fork,
   .open              = vddk_open,
   .close             = vddk_close,
   .get_size          = vddk_get_size,
