@@ -98,13 +98,20 @@ backend_load (struct backend *b, const char *name, void (*load) (void))
              program_name, b->filename, b->type);
     exit (EXIT_FAILURE);
   }
-  for (i = 0; i < len; ++i) {
+  if (! ascii_isalnum (*name)) {
+    fprintf (stderr,
+             "%s: %s: %s.name ('%s') field must begin with an "
+             "ASCII alphanumeric character\n",
+             program_name, b->filename, b->type, name);
+    exit (EXIT_FAILURE);
+  }
+  for (i = 1; i < len; ++i) {
     unsigned char c = name[i];
 
-    if (! ascii_isalnum (c)) {
+    if (! ascii_isalnum (c) && c != '-') {
       fprintf (stderr,
-               "%s: %s: %s.name ('%s') field "
-               "must contain only ASCII alphanumeric characters\n",
+               "%s: %s: %s.name ('%s') field must contain only "
+               "ASCII alphanumeric or dash characters\n",
                program_name, b->filename, b->type, name);
       exit (EXIT_FAILURE);
     }
@@ -152,12 +159,42 @@ backend_unload (struct backend *b, void (*unload) (void))
 }
 
 int
-backend_open (struct backend *b, int readonly)
+backend_list_exports (struct backend *b, int readonly, int default_only,
+                      struct nbdkit_exports *exports)
+{
+  GET_CONN;
+  struct handle *h = get_handle (conn, b->i);
+  int r;
+
+  controlpath_debug ("%s: list_exports readonly=%d default_only=%d",
+                     b->name, readonly, default_only);
+
+  assert (h->handle == NULL);
+  assert ((h->state & HANDLE_OPEN) == 0);
+  if (default_only && h->default_exportname)
+    return nbdkit_add_export (exports, h->default_exportname, NULL);
+
+  r = b->list_exports (b, readonly, default_only, exports);
+  if (r == -1)
+    controlpath_debug ("%s: list_exports failed", b->name);
+  else {
+    size_t count = nbdkit_exports_count (exports);
+    controlpath_debug ("%s: list_exports returned %zu names", b->name, count);
+    /* Best effort caching of default export name */
+    if (!h->default_exportname && count)
+      h->default_exportname = strdup (nbdkit_get_export (exports, 0).name);
+  }
+  return r;
+}
+
+int
+backend_open (struct backend *b, int readonly, const char *exportname)
 {
   GET_CONN;
   struct handle *h = get_handle (conn, b->i);
 
-  controlpath_debug ("%s: open readonly=%d", b->name, readonly);
+  controlpath_debug ("%s: open readonly=%d exportname=\"%s\" tls=%d",
+                     b->name, readonly, exportname, conn->using_tls);
 
   assert (h->handle == NULL);
   assert ((h->state & HANDLE_OPEN) == 0);
@@ -165,10 +202,24 @@ backend_open (struct backend *b, int readonly)
   if (readonly)
     h->can_write = 0;
 
+  /* Best-effort determination of the canonical name for default export */
+  if (!*exportname) {
+    if (!h->default_exportname) {
+      CLEANUP_EXPORTS_FREE struct nbdkit_exports *exps = NULL;
+
+      exps = nbdkit_exports_new (true);
+      if (exps && b->list_exports (b, readonly, true, exps) == 0 &&
+          nbdkit_exports_count (exps))
+        h->default_exportname = strdup (nbdkit_get_export (exps, 0).name);
+    }
+    if (h->default_exportname)
+      exportname = h->default_exportname;
+  }
+
   /* Most filters will call next_open first, resulting in
    * inner-to-outer ordering.
    */
-  h->handle = b->open (b, readonly);
+  h->handle = b->open (b, readonly, exportname, conn->using_tls);
   controlpath_debug ("%s: open returned handle %p", b->name, h->handle);
 
   if (h->handle == NULL) {
@@ -178,8 +229,6 @@ backend_open (struct backend *b, int readonly)
   }
 
   h->state |= HANDLE_OPEN;
-  if (b->i) /* A filter must not succeed unless its backend did also */
-    assert (get_handle (conn, b->i-1)->handle != NULL);
   return 0;
 }
 
@@ -193,9 +242,11 @@ backend_prepare (struct backend *b)
   assert ((h->state & (HANDLE_OPEN | HANDLE_CONNECTED)) == HANDLE_OPEN);
 
   /* Call these in order starting from the filter closest to the
-   * plugin, similar to typical .open order.
+   * plugin, similar to typical .open order.  But remember that
+   * a filter may skip opening its backend.
    */
-  if (b->i && backend_prepare (b->next) == -1)
+  if (b->i && get_handle (conn, b->i-1)->handle != NULL &&
+      backend_prepare (b->next) == -1)
     return -1;
 
   controlpath_debug ("%s: prepare readonly=%d", b->name, h->can_write == 0);
@@ -268,14 +319,15 @@ backend_valid_range (struct backend *b, uint64_t offset, uint32_t count)
 /* Wrappers for all callbacks in a filter's struct nbdkit_next_ops. */
 
 int
-backend_reopen (struct backend *b, int readonly)
+backend_reopen (struct backend *b, int readonly, const char *exportname)
 {
-  controlpath_debug ("%s: reopen readonly=%d", b->name, readonly);
+  controlpath_debug ("%s: reopen readonly=%d exportname=\"%s\"",
+                     b->name, readonly, exportname);
 
   if (backend_finalize (b) == -1)
     return -1;
   backend_close (b);
-  if (backend_open (b, readonly) == -1) {
+  if (backend_open (b, readonly, exportname) == -1) {
     backend_close (b);
     return -1;
   }
