@@ -61,7 +61,8 @@ run_command (void)
   if (!run)
     return;
 
-  assert (exportname != NULL);
+  if (!export_name)
+    export_name = "";
 
   fp = open_memstream (&cmd, &len);
   if (fp == NULL) {
@@ -74,16 +75,16 @@ run_command (void)
   if (port) {
     fprintf (fp, "nbd://localhost:");
     shell_quote (port, fp);
-    if (strcmp (exportname, "") != 0) {
+    if (strcmp (export_name, "") != 0) {
       putc ('/', fp);
-      uri_quote (exportname, fp);
+      uri_quote (export_name, fp);
     }
   }
   else if (unixsocket) {
     fprintf (fp, "nbd+unix://");
-    if (strcmp (exportname, "") != 0) {
+    if (strcmp (export_name, "") != 0) {
       putc ('/', fp);
-      uri_quote (exportname, fp);
+      uri_quote (export_name, fp);
     }
     fprintf (fp, "\\?socket=");
     uri_quote (unixsocket, fp);
@@ -92,7 +93,7 @@ run_command (void)
 
   /* Expose $exportname. */
   fprintf (fp, "exportname=");
-  shell_quote (exportname, fp);
+  shell_quote (export_name, fp);
   putc ('\n', fp);
 
   /* Construct older $nbd "URL".  Unfortunately guestfish and qemu take
