@@ -312,8 +312,9 @@ crypto_free (void)
  * (returns > 0), read an EOF (returns 0), or fail (returns -1).
  */
 static int
-crypto_recv (struct connection *conn, void *vbuf, size_t len)
+crypto_recv (void *vbuf, size_t len)
 {
+  GET_CONN;
   gnutls_session_t session = conn->crypto_session;
   char *buf = vbuf;
   ssize_t r;
@@ -345,18 +346,32 @@ crypto_recv (struct connection *conn, void *vbuf, size_t len)
   return 1;
 }
 
+/* If this send()'s length is so large that it is going to require
+ * multiple TCP segments anyway, there's no need to try and merge it
+ * with any corked data from a previous send that used SEND_MORE.
+ */
+#define MAX_SEND_MORE_LEN (64 * 1024)
+
 /* Write buffer to GnuTLS and either succeed completely
- * (returns 0) or fail (returns -1).
+ * (returns 0) or fail (returns -1). flags is ignored for now.
  */
 static int
-crypto_send (struct connection *conn, const void *vbuf, size_t len)
+crypto_send (const void *vbuf, size_t len, int flags)
 {
+  GET_CONN;
   gnutls_session_t session = conn->crypto_session;
   const char *buf = vbuf;
   ssize_t r;
 
   assert (session != NULL);
 
+  if (len + gnutls_record_check_corked (session) > MAX_SEND_MORE_LEN) {
+    if (gnutls_record_uncork (session, GNUTLS_RECORD_WAIT) < 0)
+      return -1;
+  }
+  else if (flags & SEND_MORE)
+    gnutls_record_cork (session);
+
   while (len > 0) {
     r = gnutls_record_send (session, buf, len);
     if (r < 0) {
@@ -368,6 +383,10 @@ crypto_send (struct connection *conn, const void *vbuf, size_t len)
     len -= r;
   }
 
+  if (!(flags & SEND_MORE) &&
+      gnutls_record_uncork (session, GNUTLS_RECORD_WAIT) < 0)
+    return -1;
+
   return 0;
 }
 
@@ -375,8 +394,9 @@ crypto_send (struct connection *conn, const void *vbuf, size_t len)
  * close, so this function ignores errors.
  */
 static void
-crypto_close (struct connection *conn)
+crypto_close (void)
 {
+  GET_CONN;
   gnutls_session_t session = conn->crypto_session;
   int sockin, sockout;
 
@@ -400,8 +420,9 @@ crypto_close (struct connection *conn)
  * only be called once per connection.
  */
 int
-crypto_negotiate_tls (struct connection *conn, int sockin, int sockout)
+crypto_negotiate_tls (int sockin, int sockout)
 {
+  GET_CONN;
   gnutls_session_t session;
   CLEANUP_FREE char *priority = NULL;
   int err;
@@ -542,7 +563,7 @@ crypto_free (void)
 }
 
 int
-crypto_negotiate_tls (struct connection *conn, int sockin, int sockout)
+crypto_negotiate_tls (int sockin, int sockout)
 {
   /* Should never be called because tls == 0. */
   abort ();
