@@ -183,16 +183,38 @@ filter_get_ready (struct backend *b)
 {
   struct backend_filter *f = container_of (b, struct backend_filter, backend);
 
-  debug ("%s: get_ready", b->name);
+  debug ("%s: get_ready thread_model=%d", b->name, thread_model);
 
   if (f->filter.get_ready) {
-    if (f->filter.get_ready (next_get_ready, b->next) == -1)
+    if (f->filter.get_ready (next_get_ready, b->next, thread_model) == -1)
       exit (EXIT_FAILURE);
   }
   else
     b->next->get_ready (b->next);
 }
 
+static int
+next_after_fork (struct backend *b)
+{
+  b->after_fork (b);
+  return 0;
+}
+
+static void
+filter_after_fork (struct backend *b)
+{
+  struct backend_filter *f = container_of (b, struct backend_filter, backend);
+
+  debug ("%s: after_fork", b->name);
+
+  if (f->filter.after_fork) {
+    if (f->filter.after_fork (next_after_fork, b->next) == -1)
+      exit (EXIT_FAILURE);
+  }
+  else
+    b->next->after_fork (b->next);
+}
+
 static int
 filter_preconnect (struct backend *b, int readonly)
 {
@@ -215,8 +237,21 @@ plugin_magic_config_key (struct backend *b)
   return b->next->magic_config_key (b->next);
 }
 
+static int
+filter_list_exports (struct backend *b, int readonly, int default_only,
+                     struct nbdkit_exports *exports)
+{
+  struct backend_filter *f = container_of (b, struct backend_filter, backend);
+
+  if (f->filter.list_exports)
+    return f->filter.list_exports (backend_list_exports, b->next,
+                                   readonly, default_only, exports);
+  return backend_list_exports (b->next, readonly, default_only, exports);
+}
+
 static void *
-filter_open (struct backend *b, int readonly)
+filter_open (struct backend *b, int readonly, const char *exportname,
+             int is_tls)
 {
   struct backend_filter *f = container_of (b, struct backend_filter, backend);
   void *handle;
@@ -225,8 +260,9 @@ filter_open (struct backend *b, int readonly)
    * inner-to-outer ordering.
    */
   if (f->filter.open)
-    handle = f->filter.open (backend_open, b->next, readonly);
-  else if (backend_open (b->next, readonly) == -1)
+    handle = f->filter.open (backend_open, b->next, readonly, exportname,
+                             is_tls);
+  else if (backend_open (b->next, readonly, exportname) == -1)
     handle = NULL;
   else
     handle = NBDKIT_HANDLE_NOT_NEEDED;
@@ -516,7 +552,9 @@ static struct backend filter_functions = {
   .config_complete = filter_config_complete,
   .magic_config_key = plugin_magic_config_key,
   .get_ready = filter_get_ready,
+  .after_fork = filter_after_fork,
   .preconnect = filter_preconnect,
+  .list_exports = filter_list_exports,
   .open = filter_open,
   .prepare = filter_prepare,
   .finalize = filter_finalize,
