@@ -76,10 +76,12 @@ static struct backend *open_filter_so (struct backend *next, size_t i, const cha
 static void start_serving (void);
 static void write_pidfile (void);
 static bool is_config_key (const char *key, size_t len);
+static void error_if_stdio_closed (void);
+static void switch_stdio (void);
 
 struct debug_flag *debug_flags; /* -D */
 bool exit_with_parent;          /* --exit-with-parent */
-const char *exportname;         /* -e */
+const char *export_name;        /* -e */
 bool foreground;                /* -f */
 const char *ipaddr;             /* -i */
 enum log_to log_to = LOG_TO_DEFAULT; /* --log */
@@ -139,6 +141,8 @@ dump_config (void)
     printf ("%s=%s\n", "binary", binary);
   printf ("%s=%s\n", "bindir", bindir);
   printf ("%s=%s\n", "filterdir", filterdir);
+  printf ("%s=%s\n", "host_cpu", host_cpu);
+  printf ("%s=%s\n", "host_os", host_os);
   printf ("%s=%s\n", "libdir", libdir);
   printf ("%s=%s\n", "mandir", mandir);
   printf ("%s=%s\n", "name", PACKAGE_NAME);
@@ -159,6 +163,11 @@ dump_config (void)
   printf ("%s=%s\n", "version", PACKAGE_VERSION);
   printf ("%s=%d\n", "version_major", NBDKIT_VERSION_MAJOR);
   printf ("%s=%d\n", "version_minor", NBDKIT_VERSION_MINOR);
+#ifdef HAVE_LIBZSTD
+  printf ("zstd=yes\n");
+#else
+  printf ("zstd=no\n");
+#endif
 }
 
 int
@@ -177,16 +186,7 @@ main (int argc, char *argv[])
   size_t i;
   const char *magic_config_key;
 
-  /* Refuse to run if stdin/out/err are closed, whether or not -s is used. */
-  if (fcntl (STDERR_FILENO, F_GETFL) == -1) {
-    /* Nowhere we can report the error. Oh well. */
-    exit (EXIT_FAILURE);
-  }
-  if (fcntl (STDIN_FILENO, F_GETFL) == -1 ||
-      fcntl (STDOUT_FILENO, F_GETFL) == -1) {
-    perror ("expecting stdin/stdout to be opened");
-    exit (EXIT_FAILURE);
-  }
+  error_if_stdio_closed ();
 
 #if !ENABLE_LIBFUZZER
   threadlocal_init ();
@@ -340,13 +340,7 @@ main (int argc, char *argv[])
       break;
 
     case 'e':
-      exportname = optarg;
-      if (strnlen (exportname, NBD_MAX_STRING + 1) > NBD_MAX_STRING) {
-        nbdkit_error ("export name too long");
-        exit (EXIT_FAILURE);
-      }
-      /* TODO: Check that name is valid UTF-8? */
-      newstyle = true;
+      export_name = optarg;
       break;
 
     case 'f':
@@ -482,18 +476,6 @@ main (int argc, char *argv[])
     exit (EXIT_FAILURE);
   }
 
-  /* Oldstyle protocol + exportname not allowed. */
-  if (!newstyle && exportname != NULL) {
-    fprintf (stderr,
-             "%s: cannot use oldstyle protocol (-o) and exportname (-e)\n",
-             program_name);
-    exit (EXIT_FAILURE);
-  }
-
-  /* If exportname was not set on the command line, use "". */
-  if (exportname == NULL)
-    exportname = "";
-
   /* --tls=require and oldstyle won't work. */
   if (tls == 2 && !newstyle) {
     fprintf (stderr,
@@ -605,7 +587,7 @@ main (int argc, char *argv[])
   }
 
   /* Apply nbdkit.* flags for the server. */
-  apply_debug_flags (NULL, "nbdkit");
+  apply_debug_flags (RTLD_DEFAULT, "nbdkit");
 
   /* Check all debug flags were used, and free them. */
   free_debug_flags ();
@@ -706,46 +688,14 @@ main (int argc, char *argv[])
   /* Select the correct thread model based on config. */
   lock_init_thread_model ();
 
-  /* Sanitize stdin/stdout to /dev/null, after saving the originals
-   * when needed.  We are still single-threaded at this point, and
-   * already checked that stdin/out were open, so we don't have to
-   * worry about other threads accidentally grabbing our intended fds,
-   * or races on FD_CLOEXEC.  POSIX says that 'fflush(NULL)' is
-   * supposed to reset the underlying offset of seekable stdin, but
-   * glibc is buggy and requires an explicit fflush(stdin) as
-   * well. https://sourceware.org/bugzilla/show_bug.cgi?id=12799
-   */
-  fflush (stdin);
-  fflush (NULL);
-  if (listen_stdin || run) {
-#ifndef F_DUPFD_CLOEXEC
-#define F_DUPFD_CLOEXEC F_DUPFD
-#endif
-    saved_stdin = fcntl (STDIN_FILENO, F_DUPFD_CLOEXEC, STDERR_FILENO + 1);
-    saved_stdout = fcntl (STDOUT_FILENO, F_DUPFD_CLOEXEC, STDERR_FILENO + 1);
-#if F_DUPFD == F_DUPFD_CLOEXEC
-    saved_stdin = set_cloexec (saved_stdin);
-    saved_stdout = set_cloexec (saved_stdout);
-#endif
-    if (saved_stdin == -1 || saved_stdout == -1) {
-      perror ("fcntl");
-      exit (EXIT_FAILURE);
-    }
-  }
-  close (STDIN_FILENO);
-  close (STDOUT_FILENO);
-  if (open ("/dev/null", O_RDONLY) != STDIN_FILENO ||
-      open ("/dev/null", O_WRONLY) != STDOUT_FILENO) {
-    perror ("open");
-    exit (EXIT_FAILURE);
-  }
-
   /* Tell the plugin that we are about to start serving.  This must be
    * called before we change user, fork, or open any sockets.
    */
-  configured = true;
   top->get_ready (top);
 
+  switch_stdio ();
+  configured = true;
+
   start_serving ();
 
   top->free (top);
@@ -954,6 +904,7 @@ start_serving (void)
     debug ("using socket activation, nr_socks = %zu", socks.size);
     change_user ();
     write_pidfile ();
+    top->after_fork (top);
     accept_incoming_connections (&socks);
     return;
   }
@@ -962,6 +913,7 @@ start_serving (void)
   if (listen_stdin) {
     change_user ();
     write_pidfile ();
+    top->after_fork (top);
     threadlocal_new_server_thread ();
     handle_single_connection (saved_stdin, saved_stdout);
     return;
@@ -981,6 +933,7 @@ start_serving (void)
   change_user ();
   fork_into_background ();
   write_pidfile ();
+  top->after_fork (top);
   accept_incoming_connections (&socks);
 }
 
@@ -1042,3 +995,56 @@ is_config_key (const char *key, size_t len)
 
   return true;
 }
+
+/* Refuse to run if stdin/out/err are closed, whether or not -s is used. */
+static void
+error_if_stdio_closed (void)
+{
+  if (fcntl (STDERR_FILENO, F_GETFL) == -1) {
+    /* Nowhere we can report the error. Oh well. */
+    exit (EXIT_FAILURE);
+  }
+  if (fcntl (STDIN_FILENO, F_GETFL) == -1 ||
+      fcntl (STDOUT_FILENO, F_GETFL) == -1) {
+    perror ("expecting stdin/stdout to be opened");
+    exit (EXIT_FAILURE);
+  }
+}
+
+/* Sanitize stdin/stdout to /dev/null, after saving the originals
+ * when needed.  We are still single-threaded at this point, and
+ * already checked that stdin/out were open, so we don't have to
+ * worry about other threads accidentally grabbing our intended fds,
+ * or races on FD_CLOEXEC.  POSIX says that 'fflush(NULL)' is
+ * supposed to reset the underlying offset of seekable stdin, but
+ * glibc is buggy and requires an explicit fflush(stdin) as
+ * well. https://sourceware.org/bugzilla/show_bug.cgi?id=12799
+ */
+static void
+switch_stdio (void)
+{
+  fflush (stdin);
+  fflush (NULL);
+  if (listen_stdin || run) {
+#ifndef F_DUPFD_CLOEXEC
+#define F_DUPFD_CLOEXEC F_DUPFD
+#endif
+    saved_stdin = fcntl (STDIN_FILENO, F_DUPFD_CLOEXEC, STDERR_FILENO + 1);
+    saved_stdout = fcntl (STDOUT_FILENO, F_DUPFD_CLOEXEC, STDERR_FILENO + 1);
+#if F_DUPFD == F_DUPFD_CLOEXEC
+    saved_stdin = set_cloexec (saved_stdin);
+    saved_stdout = set_cloexec (saved_stdout);
+#endif
+    if (saved_stdin == -1 || saved_stdout == -1) {
+      perror ("fcntl");
+      exit (EXIT_FAILURE);
+    }
+  }
+  close (STDIN_FILENO);
+  close (STDOUT_FILENO);
+  if (open ("/dev/null", O_RDONLY) != STDIN_FILENO ||
+      open ("/dev/null", O_WRONLY) != STDOUT_FILENO) {
+    perror ("open");
+    exit (EXIT_FAILURE);
+  }
+}
