@@ -158,8 +158,11 @@ plugin_dump_fields (struct backend *b)
   HAS (config);
   HAS (config_complete);
   HAS (config_help);
+  HAS (thread_model);
   HAS (get_ready);
+  HAS (after_fork);
   HAS (preconnect);
+
   HAS (open);
   HAS (close);
   HAS (get_size);
@@ -167,25 +170,26 @@ plugin_dump_fields (struct backend *b)
   HAS (can_flush);
   HAS (is_rotational);
   HAS (can_trim);
-  HAS (_pread_v1);
-  HAS (_pwrite_v1);
-  HAS (_flush_v1);
-  HAS (_trim_v1);
-  HAS (_zero_v1);
   HAS (can_zero);
+  HAS (can_fast_zero);
+  HAS (can_extents);
   HAS (can_fua);
+  HAS (can_multi_conn);
+  HAS (can_cache);
+
   HAS (pread);
   HAS (pwrite);
   HAS (flush);
   HAS (trim);
   HAS (zero);
-  HAS (can_multi_conn);
-  HAS (can_extents);
   HAS (extents);
-  HAS (can_cache);
   HAS (cache);
-  HAS (thread_model);
-  HAS (can_fast_zero);
+
+  HAS (_pread_v1);
+  HAS (_pwrite_v1);
+  HAS (_flush_v1);
+  HAS (_trim_v1);
+  HAS (_zero_v1);
 #undef HAS
 
   /* Custom fields. */
@@ -249,6 +253,20 @@ plugin_get_ready (struct backend *b)
     exit (EXIT_FAILURE);
 }
 
+static void
+plugin_after_fork (struct backend *b)
+{
+  struct backend_plugin *p = container_of (b, struct backend_plugin, backend);
+
+  debug ("%s: after_fork", b->name);
+
+  if (!p->plugin.after_fork)
+    return;
+
+  if (p->plugin.after_fork () == -1)
+    exit (EXIT_FAILURE);
+}
+
 static int
 plugin_preconnect (struct backend *b, int readonly)
 {
@@ -262,14 +280,48 @@ plugin_preconnect (struct backend *b, int readonly)
   return p->plugin.preconnect (readonly);
 }
 
-static void *
-plugin_open (struct backend *b, int readonly)
+static int
+plugin_list_exports (struct backend *b, int readonly, int default_only,
+                     struct nbdkit_exports *exports)
 {
+  return nbdkit_add_export (exports, "", NULL);
+}
+
+static void *
+plugin_open (struct backend *b, int readonly, const char *exportname,
+             int is_tls)
+{
+  GET_CONN;
   struct backend_plugin *p = container_of (b, struct backend_plugin, backend);
+  void *r;
 
   assert (p->plugin.open != NULL);
+  assert (conn->exportname == NULL);
 
-  return p->plugin.open (readonly);
+  /* Save the exportname since the lifetime of the string passed in
+   * here is likely to be brief.  In addition this provides a place
+   * for nbdkit_export_name to retrieve it if called from the plugin.
+   * While readonly and the export name can be altered in plugins, the
+   * tls mode cannot.
+   *
+   * In API V3 we propose to pass the exportname and tls mode as extra
+   * parameters to the (new) plugin.open and deprecate
+   * nbdkit_export_name and nbdkit_is_tls for V3 users.  Even then we
+   * will still need to save the export name in the handle because of
+   * the lifetime issue.
+   */
+  conn->exportname = strdup (exportname);
+  if (conn->exportname == NULL) {
+    nbdkit_error ("strdup: %m");
+    return NULL;
+  }
+
+  r = p->plugin.open (readonly);
+  if (r == NULL) {
+    free (conn->exportname);
+    conn->exportname = NULL;
+  }
+  return r;
 }
 
 /* We don't expose .prepare and .finalize to plugins since they aren't
@@ -292,10 +344,13 @@ plugin_finalize (struct backend *b, void *handle)
 static void
 plugin_close (struct backend *b, void *handle)
 {
+  GET_CONN;
   struct backend_plugin *p = container_of (b, struct backend_plugin, backend);
 
   if (handle && p->plugin.close)
     p->plugin.close (handle);
+  free (conn->exportname);
+  conn->exportname = NULL;
 }
 
 static int64_t
@@ -617,7 +672,6 @@ plugin_zero (struct backend *b, void *handle,
     goto done;
   }
 
-  assert (p->plugin.pwrite || p->plugin._pwrite_v1);
   flags &= ~NBDKIT_FLAG_MAY_TRIM;
   threadlocal_set_error (0);
   *err = 0;
@@ -651,8 +705,10 @@ plugin_extents (struct backend *b, void *handle,
   struct backend_plugin *p = container_of (b, struct backend_plugin, backend);
   int r;
 
-  /* This should be true because plugin_can_extents checks it. */
-  assert (p->plugin.extents);
+  if (p->plugin.extents == NULL) { /* Possible if .can_extents lied */
+    *err = EINVAL;
+    return -1;
+  }
 
   r = p->plugin.extents (handle, count, offset, flags, extents);
   if (r >= 0 && nbdkit_extents_count (extents) < 1) {
@@ -695,7 +751,9 @@ static struct backend plugin_functions = {
   .config_complete = plugin_config_complete,
   .magic_config_key = plugin_magic_config_key,
   .get_ready = plugin_get_ready,
+  .after_fork = plugin_after_fork,
   .preconnect = plugin_preconnect,
+  .list_exports = plugin_list_exports,
   .open = plugin_open,
   .prepare = plugin_prepare,
   .finalize = plugin_finalize,
