@@ -31,12 +31,12 @@
 # SUCH DAMAGE.
 
 # Test the full plugin.  Note that this plugin causes libguestfs to
-# hang so we have to use low level tools (qemu-io).
+# hang so we have to use low level tools.
 
 source ./functions.sh
 set -e
 
-requires qemu-io --version
+requires nbdsh --version
 
 sock=`mktemp -u`
 files="full.pid $sock full.out"
@@ -44,16 +44,26 @@ rm -f $files
 cleanup_fn rm -f $files
 
 # Run nbdkit with the full plugin.
-start_nbdkit -P full.pid -U $sock full size=1M
+start_nbdkit -P full.pid -U $sock full 1M
 
 # All reads should succeed.
-qemu-io -f raw "nbd+unix://?socket=$sock" \
-        -c 'r -v 0 512' \
-        -c 'r -v 512 512' \
-        -c 'r -v 1048064 512'
+nbdsh --connect "nbd+unix://?socket=$sock" \
+      -c 'h.pread(512, 0)' \
+      -c 'h.pread(512, 512)' \
+      -c 'h.pread(512, 1048064)'
 
 # All writes should fail with the ENOSPC error.
-! LANG=C qemu-io -f raw "nbd+unix://?socket=$sock" \
-        -c 'w -P 1 0 512' \
-        -c 'w -P 2 1048064 512' >& full.out
-grep "No space left on device" full.out
+nbdsh --connect "nbd+unix://?socket=$sock" \
+      -c '
+def test(offset):
+    try:
+        h.pwrite(bytearray(512), offset)
+        # This should not happen.
+        exit(1)
+    except nbd.Error as ex:
+        # Check the errno is expected.
+        assert ex.errno == "ENOSPC"
+
+test(0)
+test(1048064)
+'
