@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
-# Copyright (C) 2019 Red Hat Inc.
+# Copyright (C) 2019-2020 Red Hat Inc.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -49,10 +49,6 @@ name1k=$name256$name256$name256$name256
 name4k=$name1k$name1k$name1k$name1k
 almost4k=${name4k%8$name16}
 
-# Test behavior of -e: accept 4k max, reject longer
-nbdkit -U - -e $name4k null --run true || fail=1
-nbdkit -U - -e a$name4k null --run true && fail=1
-
 # Test that $exportname and $uri reflect the name
 out=$(nbdkit -U - -e $name4k null --run 'echo $exportname')
 if test "$name4k" != "$out"; then
@@ -95,24 +91,17 @@ nbdkit -U - -e $almost4k null --run 'export exportname uri
 nbdsh -c - <<\EOF
 import os
 long = os.environ["exportname"]
-h.set_export_name (long)
-h.add_meta_context ("a" + long)
-h.add_meta_context ("b" + long)
-h.add_meta_context ("c" + long)
-h.add_meta_context ("d" + long)
-h.add_meta_context ("e" + long)
-h.connect_uri (os.environ["uri"])
+h.set_export_name(long)
+h.add_meta_context("a" + long)
+h.add_meta_context("b" + long)
+h.add_meta_context("c" + long)
+h.add_meta_context("d" + long)
+h.add_meta_context("e" + long)
+h.connect_uri(os.environ["uri"])
 assert h.get_size() == 0
 EOF
 '
 
-# The rest of this test uses the ‘qemu-nbd --list’ option added in qemu 4.0.
-if ! qemu-nbd --help | grep -sq -- --list; then
-    echo "$0: skipping because qemu-nbd does not support the --list option"
-    exit 77
-fi
-
-# Test response to NBD_OPT_LIST
-nbdkit -U - -e $name4k null --run 'qemu-nbd --list -k $unixsocket' || fail=1
+# See also test-eval-exports.sh for NBD_OPT_LIST with long name
 
 exit $fail
